unit MultiBaseEdit;

{ Copyright (c) 1996 David Novak

	Version 1.0, 12-16-96:
    New release

  Note from author:
    It is not required to notify me of uses of MultiBaseEdit, but it would be
    very interesting to know where it is being used. Also, please send me an
    email if you find a bug or have a suggestion for an enhancement. If you make
    changed to the code, email the code to me for possible inclusion in the next
    version.

    Thanks for using MultiBaseEdit,
    David Novak
    novak@valu-line.net

  Description:
    MultiBaseEdit behaves as an edit box, except that it only allows integers
    to be typed into it. The integers may be interpreted as decimal,
    hexadecimal, or binary.

  Properties:
    All properties of TEdit.

    Base      (TBase)
    This specifies how the values typed in the edit box should be interpretted.
    Possible values are bDEC, bHEX, and bBIN, which correspond to decimal,
    hexadecimal, and binary, respectfully. Bounds checking is implemented so
    that only valid digits are allowed to be typed into the MultiBaseEdit
    control.

    Value     (LongInt)
    This is the interpretted value of the text in the MultiBaseEdit.

  Methods:
    All the methods of TEdit.

  Events:
    All the events of TEdit.

  License:
    MultiBaseEdit is freeware. It may be freely used for any purpose. You may
    use it privately or in the course of your work; there is no fee, and no
    registration is required. You may distribute it to anyone, and you may place
    it on any archive or bulletin board system. You may not charge anyone for it
    other than a reasonable fee to cover your distribution costs. Normally, you
    should distribute MultiBaseEdit in the form as supplied by the author;
    however, you may repackage it to suit the conventions and needs of an
    archive or bulletin board system.

  Disclaimer:
    MultiBaseEdit is supplied on an as-is basis. The author offers no warranty
    of its fitness for any purpose whatsoever, and accepts no liability
    whatsoever for any loss or damage incurred by its use.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

const
	DecDigits : set of '0'..'9' =['0'..'9'];
  BinDigits : set of '0'..'1' =['0'..'1'];
  HexDigits : set of '0'..'f' =['0'..'9', 'A'..'F', 'a'..'f'];
  UpCaseHexLetters : set of 'A'..'F' =['A'..'F'];
  LowCaseHexLetters : set of 'a'..'f' =['a'..'f'];

type
	TBase = ( bDEC, bHEX, bBIN );

type
  TMultiBaseEdit = class(TCustomEdit)
  private
    FBase : TBase;
    FValue : LongInt;
    procedure SetBase( b : TBase );
    procedure SetValue( v : LongInt );
    function GetValue : LongInt;
    function GetBinString( v : LongInt ) : string;
    function GetHexString( v : LongInt ) : string;
    function BinStrToInt( s : string ) : LongInt;
    function HexStrToInt( s : string ) : LongInt;
    function StripLeadingZeros( s : string ) : string;
  protected
		procedure KeyPress( var key : char ); override;
  public
		constructor Create(AOwner: TComponent); override;
  published
    property AutoSelect;
    property AutoSize;
    property Base : TBase read FBase write SetBase default bDEC;
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property HideSelection;
    property MaxLength;
    property OEMConvert;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Value : LongInt read GetValue write SetValue default 0;
    property Visible;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDrag;
  end;

procedure Register;

implementation

procedure TMultiBaseEdit.KeyPress( var key : char );
var
	KeyOk : boolean;
begin
	inherited KeyPress( key );

  KeyOk :=false;

  case FBase of
  	bDEC : KeyOk :=(key in DecDigits) or ((key = '-') and (SelStart = 0));
    bHEX : KeyOk :=key in HexDigits;
    bBIN : KeyOk :=key in BinDigits;
  end;

  // backspace is a special case
  if key = #8 then
  	KeyOk :=true;

  if not KeyOk then
  	key :=#0;
end;

// Convert a binary string into a LongInt value. s must be no larger than
// 32 characters. Each character of s should be either a 1 or a 0.
function TMultiBaseEdit.BinStrToInt( s : string ) : LongInt;
var
	i : integer;
  v : LongInt;
begin
	v :=0;

  if length(s) > 32 then
  begin
  	BinStrToInt :=0;
  	exit;
  end;

  for i :=1 to length(s) do
  begin
  	{$R-}
  	v :=v shl 1;
    {$R+}

  	if s[i] = '1' then
    	v :=v or 1;
  end;

  BinStrToInt :=v;
end;

// Convert a hexadecimal string into a LongInt value. s must be no larger than
// 8 characters. Each character of s should be either a 0..9 or A..F or a..f.
function TMultiBaseEdit.HexStrToInt( s : string ) : LongInt;
const
	HexVals : array [0..$F] of integer =( 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
  	$A, $B, $C, $D, $E, $F );
var
	v : LongInt;
  i : integer;
  LookUpIndex : integer;
begin
	if length(s) > 8 then
  begin
  	HexStrToInt :=0;
    exit;
  end;

  v :=0;
	for i :=1 to length(s) do
  begin
  	{$R-}
  	v :=v shl 4;
    {$R+}

  	if s[i] in DecDigits then
    	LookUpIndex :=ord(s[i])-ord('0')
    else if s[i] in UpCaseHexLetters then
    	LookUpIndex :=ord(s[i])-ord('A') + $A
    else if s[i] in LowCaseHexLetters then
    	LookUpIndex :=ord(s[i])-ord('a') + $A
    else
    	LookUpIndex :=0;

    v :=v or HexVals[LookUpIndex];
  end;

  HexStrToInt :=v;
end;

constructor TMultiBaseEdit.Create(AOwner: TComponent);
begin
	inherited create( AOwner );

  FBase :=bDEC;
  value :=0;
end;

procedure TMultiBaseEdit.SetBase( b : TBase );
begin
	// update FValue, just in case somebody typed into edit control
  FValue :=GetValue;

  // change the base and display the new string
  FBase :=b;
  SetValue( FValue );
end;

procedure TMultiBaseEdit.SetValue( v : LongInt );
begin
	FValue :=v;

  case FBase of
  	bDEC : text :=IntToStr( v );
    bHEX : text :=GetHexString( v );
    bBIN : text :=GetBinString( v );
  end;
end;

function TMultiBaseEdit.GetValue : LongInt;
var
	v : LongInt;
  s : string;
begin
	// error check
  if text = '' then
  begin
  	GetValue :=0;
    exit;
  end;

  s :=StripLeadingZeros( text );

	v :=0;

	case FBase of
  	bDEC : v :=StrToInt( s );
    bHEX : v :=HexStrToInt( s );
    bBIN : v :=BinStrToInt( s );
  end;

  GetValue :=v;
end;

function TMultiBaseEdit.StripLeadingZeros( s : string ) : string;
var
  i : integer;
  len : integer;
  DestString : string;
begin
  len :=length(s);

  for i :=1 to len do
    if s[i] <> '0' then
      break;

  // special case for a string of zeros
  if i > len then
    result :='0'
  else
    result :=copy( s, i, len-i+1 );
end;

// This function returns a string of 1 and 0's which is the
// binary representation of v.
function TMultiBaseEdit.GetBinString( v : LongInt ) : string;
var
  i : Integer;
  mask : LongInt;
  s : string;
begin
  mask := $00000001;
  s := '';
  for i := 1 to 32 do
  begin
    if (v And mask) <> 0 then
      S := '1' + s
    else
      s := '0' + s;

    {$R-}
    mask := mask shl 1;
    {$R+}
  end;

  GetBinString := s;
end;

// Get the hexadecimal string equivalent of v
function TMultiBaseEdit.GetHexString( v : LongInt ) : string;
const
  HexChars: array [0..$F] of Char =  '0123456789ABCDEF';
var
  s : string;
  i : integer;
  c : char;
begin
	s :='';

  for i :=1 to 8 do
  begin
  	c :=HexChars[v and $0000000F];
    s := c + s;

    v :=v shr 4;
  end;

	GetHexString :=s;
end;

procedure Register;
begin
  RegisterComponents('FreeWare', [TMultiBaseEdit]);
end;

end.

